<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/10/2008 *
 * ===================                          Last change: 10/10/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : yoomedia_functions.php                           *
 * -------------------------------------------------------------------- *
 * Short description : Special functions for yoomedia extension         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Spezielle Funktion fuer Yoo!Media-Erweiterung    *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009, 2010 by Mailer Developer Team                    *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Queries the given Yoo!Media API 2.0 script
function YOOMEDIA_QUERY_API ($script, $countQuery = true) {
	// Init response array
	$response = array();

	// Enougth queries left?
	if ((getConfig('yoomedia_requests_remain') > 0) || ($countQuery === false)) {
		// Prepare the low-level request
		$requestString = sprintf("http://www.yoomedia.de/interface_2.0/%s?id=%s&sid=%s&pw=%s&reload=%s&ma=%s&uebrig=%s&verguetung=%s&erotik=%s",
			$script,
			getConfig('yoomedia_id'),
			getConfig('yoomedia_sid'),
			getConfig('yoomedia_passwd'),
			getConfig('yoomedia_tm_max_reload'),
			getConfig('yoomedia_tm_min_wait'),
			getConfig('yoomedia_tm_clicks_remain'),
			getConfig('yoomedia_tm_min_pay'),
			getConfig('yoomedia_erotic_allowed')
		);

		// Run the query
		$response = sendGetRequest($requestString);

		// Convert from ISO to UTF-8 only if count is > 3 because <= 3 means timeout
		if (count($response) > 3) {
			// Convert all lines to UTF-8
			foreach ($response as $k => $v) {
				// Convert the line
				$response[$k] = iconv('windows-1252', 'UTF-8//TRANSLIT', $v);
				/*
				// iconv()-less ISO-8859-1 -> UTF-8
				$response[$k] = preg_replace(
					"/([\x80-\xFF])/e",
					"chr(0xC0|ord('\\1')>>6).chr(0x80|ord('\\1')&0x3F)",
					$v
				);
				*/
			} // END - foreach
		} // END - if

		// Shall we count the query as used?
		if ($countQuery === true) {
			// Then update the config!
			updateConfiguration('yoomedia_requests_remain', 1, '-');
		} // END - if
	} // END - if

	// Return the data
	return $response;
}

// Test if the extension settings did work
function YOOMEDIA_TEST_CONFIG ($data) {
	// Is this admin?
	if (!isAdmin()) {
		// No admin!
		return false;
	} // END - if

	// Transfer config data
	mergeConfig($data);

	// Temporary allow maximum
	setConfigEntry('yoomedia_tm_max_reload'   , 100000);
	setConfigEntry('yoomedia_tm_min_wait'     , 0);
	setConfigEntry('yoomedia_tm_clicks_remain', 10);
	setConfigEntry('yoomedia_tm_min_pay'      , 0);
	setConfigEntry('yoomedia_erotic_allowed'  , 1);

	// Query the API with a test request without couting it
	// If zero reply comes back the data is invalid!
	$response = YOOMEDIA_QUERY_API('out_textmail.php', true); // @TODO Ask Yoo!Media for test script

	// Default error code is 0 = all fine!
	$errorCode = YOOMEDIA_GET_ERRORCODE_FROM_RESULT($response);

	// Log the response if failed
	if (count($response) == 0) {
		// Queries depleted (as we count here!)
		logDebugMessage(__FUNCTION__, __LINE__, 'Requested depleted. Maxmimum was: ' . getConfig('yoomedia_requests_total'));
		$errorCode = -1;
	} elseif (!isset($response[8])) {
		// Invalid response
		logDebugMessage(__FUNCTION__, __LINE__, 'Missing response line [8]. Raw response=' . base64_encode(serialize($response)));
		$errorCode = -1;
	} elseif ((($errorCode <= 4) && ($errorCode > 0)) || ($errorCode >= 8)) {
		// An error has returned from the account
		logDebugMessage(__FUNCTION__, __LINE__, 'Unexpected error code ' . $errorCode . ' received.');
	} elseif (count($response) < 9) {
		// Log serialized raw response
		logDebugMessage(__FUNCTION__, __LINE__, 'Raw response=' . base64_encode(serialize($response)));
		$errorCode = -1;
	} else {
		// This is fine, because the result array is okay and the response code on element 8 is fine
		$errorCode = '0';
	}

	// Do we have some data there?
	return ($errorCode == '0');
}

// "Getter" for a parsed result for all text mails. This means an array without
// the header lines will be returned
function YOOMEDIA_GET_PARSED_RESULT_TEXTMAILS () {
	// Get the raw response
	$response = YOOMEDIA_QUERY_API('out_textmail.php');

	// Parse the response
	$result = YOOMEDIA_PARSE_RESPONSE($response, 'textmail');

	// Return result
	return $result;
}

// Parser function for Yoo!Media API responses
function YOOMEDIA_PARSE_RESPONSE ($response, $type) {
	// Init result
	$result = array();

	// Cut off the header
	$dummy = $response;
	foreach ($response as $line) {
		// Remove line
		array_shift($dummy);

		// Is this line empty?
		if (empty($line)) {
			// Then stop here
			break;
		} // END - if
	} // END - foreach

	// If we have no result, abort here
	if (count($dummy) == 0) {
		// Empty response from API
		logDebugMessage(__FUNCTION__, __LINE__, 'Empy result from API received.');
		return array();
	} // END - if

	// The result is now still raw, so we must split it up and trim spaces away
	$responseLine = trim(implode("\n", $dummy));

	// Last line should never be a pipe!
	if (substr($responseLine, -1, 1) == '|') $responseLine = substr($responseLine, 0, -1);

	// Now, explode all in one array
	$dataArray = explode('|', $responseLine);

	// Now make the result array with two dimensions
	$cnt = '0'; $entry = '0';
	foreach ($dataArray as $line) {
		// Add the line
		$result[$entry][yoomediaTranslateIndex($type, $cnt)] = $line;

		// End of data of first entry reached?
		if ($cnt == 6) {
			// Then advance to next entry and reset counter
			$entry++;
			$cnt = '0';
		} else {
			// Count up
			$cnt++;
		}
	} // END - foreach

	// Return it
	return $result;
}

// Prepares a bonus mail for delivery. Works only if extension 'bonus' is active
function YOOMEDIA_PREPARE_MAIL_DELIVERY ($data) {
	// Is this an admin?
	if (!isAdmin()) {
		// Abort here
		return false;
	} elseif (!isExtensionActive('bonus')) {
		// Abort here
		return false;
	}

	// Is the waiting time below one second? Then fix it to one (zero seconds are not yet supported!)
	if ($data['wait'] < 1) $data['wait'] = 1;

	// Half of waiting time is a good reward!
	$data['reward'] = round($data['wait'] / 2 + 0.4);

	// Is the reward below one?
	if ($data['reward'] < 1) $data['reward'] = 1;

	// Add website id
	$data['sid'] = getConfig('yoomedia_sid');

	// Add total receivers
	$data['all'] = translateComma(getTotalReceivers());

	// Add categories
	$data['categories'] = generateCategoryOptionsList('normal');

	// Decode entities
	$data['text'] = decodeEntities($data['text']);

	// Load template
	loadTemplate('admin_send_yoomedia', false, $data);
}

// Adds the mail to the bonus mail pool
function YOOMEDIA_SEND_BONUS_MAIL ($data, $mode) {
	// Is this an admin?
	if (!isAdmin()) {
		// Abort here
		return false;
	} elseif (!isExtensionActive('bonus')) {
		// Abort here
		return false;
	}

	// Add dummy receiver to avoid notice
	$data['receiver'] = '0';

	// HTML or normal? (normal is default...)
	$type = 't';
	if (($mode == 'html') && (isExtensionActive('html_mail'))) $type = 'h';

	// Auto-generate URL
	$data['url'] = sprintf("http://www.yoomedia.de/code/%s-mail.php?id=%s&sid=%s",
		$type,
		$data['id'],
		$data['sid']
	);

	// Lock this mail for new delivery
	YOOMEDIA_RELOAD_LOCK($data, $mode);

	// Call the lower function
	addNewBonusMail($data, $mode);
}

// Lockdown given id
function YOOMEDIA_EXCLUDE_MAIL ($data, $mode) {
	// Search for the entry
	if (YOOMEDIA_CHECK_RELOAD($data['id'], $data['reload'], $mode) === false) {
		// Convert mode for mails
		$mode = YOOMEDIA_CONVERT_MODE($mode);

		// Add the entry
		SQL_QUERY_ESC("INSERT INTO `{?_MYSQL_PREFIX?}_yoomedia_reload` (`type`,`y_id`,`y_reload`,`inserted`) VALUES ('%s',%s,%s,'0000-00-00 00:00')",
			array(
				$mode,
				bigintval($data['id']),
				bigintval($data['reload'])
			), __FUNCTION__, __LINE__);
	} // END - if
}

// Remove lock of given mail
function YOOMEDIA_UNLIST_MAIL ($data, $mode) {
	// Convert mode for mails
	$mode = YOOMEDIA_CONVERT_MODE($mode);

	// Add the entry
	SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_yoomedia_reload` WHERE `type`='%s' AND `y_id`=%s LIMIT 1",
		array($mode, bigintval($data['id'])), __FUNCTION__, __LINE__);
}

// "Translates" the index number into an assosiative value
function yoomediaTranslateIndex ($type, $index) {
	// Default is the index
	$return = $index;

	// Is the element there?
	if (isset($GLOBALS['translation_tables']['yoomedia'][$type][$index])) {
		// Use this element
		$return = $GLOBALS['translation_tables']['yoomedia'][$type][$index];
	} else {
		// Not found!
		logDebugMessage(__FUNCTION__, __LINE__, "type={$type},index={$index} not found.");
	}

	// Return value
	return $return;
}

// "Translate" error code
function translateYooMediaError ($errorCode) {
	// Default is 'failed'
	$return = 'failed (Code: ' . $errorCode . ')';

	// Is the entry there?
	if (isset($GLOBALS['translation_tables']['yoomedia']['error_codes'][$errorCode])) {
		// Entry found!
		$return = $GLOBALS['translation_tables']['yoomedia']['error_codes'][$errorCode];
	} else {
		// Log missing entries
		debug_report_bug(__FUNCTION__, __LINE__, sprintf("Unknown error code <strong>%s[%s]</strong> detected.", $errorCode, gettype($errorCode)));
	}

	// Return value
	return $return;
}

// Checks if the mail id is in reload lock
function YOOMEDIA_CHECK_RELOAD ($id, $reload, $type) {
	// Default is not in reload lock
	$reloaded = false;

	// Query database
	$result = SQL_QUERY_ESC("SELECT `id`, UNIX_TIMESTAMP(`inserted`) AS inserted FROM `{?_MYSQL_PREFIX?}_yoomedia_reload` WHERE `type`='%s' AND `y_id`=%s LIMIT 1",
		array($type, bigintval($id)), __FUNCTION__, __LINE__);

	// Entry found?
	if (SQL_NUMROWS($result) == 1) {
		// Load time
		list($id, $time) = SQL_FETCHROW($result);

		// Are we ready to sent again?
		if (((time() - $time) >= ($reload * 60*60)) && ($time > 0)) {
			// Remove entry
			SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_yoomedia_reload` WHERE `id`=%s LIMIT 1",
				array($id), __FUNCTION__, __LINE__);
		} else {
			// Dont' sent again this mail
			$reloaded = $time;
		}
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $reloaded;
}

// Lock given mail down for reload lock
function YOOMEDIA_RELOAD_LOCK ($data, $mode) {
	// Search for the entry
	if (YOOMEDIA_CHECK_RELOAD($data['id'], $data['reload'], $mode) === false) {
		// Convert mode for mails
		$mode = YOOMEDIA_CONVERT_MODE($mode);

		// Add the entry
		SQL_QUERY_ESC("INSERT INTO `{?_MYSQL_PREFIX?}_yoomedia_reload` (`type`,`y_id`,`y_reload`) VALUES ('%s',%s,%s)",
			array($mode, bigintval($data['id']), bigintval($data['reload'])), __FUNCTION__, __LINE__);
	} // END - if
}

// Convert mode for mails
function YOOMEDIA_CONVERT_MODE ($mode) {
	// Convert mode for normal/html
	switch ($mode) {
		case 'normal':
			$mode = 'textmail';
			break;

		case 'html':
			$mode = 'htmlmail';
			break;
	} // END - switch

	// Return result
	return $mode;
}

// Extract code from response
function YOOMEDIA_GET_ERRORCODE_FROM_RESULT ($response) {
	// Bad code as default
	$code = -999;

	// Which response should we parse?
	if ((isset($response[8])) && (count($response) == 9)) {
		// Use error code from element 8 (mostly API errors)
		$codeArray = explode('<br>', $response[8]);

		// Use only the first element
		$code = bigintval($codeArray[0]);
	} elseif ((is_array($response[0])) && (isset($response[0]['id']))) {
		// Begin with extraction
		$codeArray = explode(' ', $response[0]['id']);
		$code = $codeArray[0];
		$codeArray = explode('<br />', $code);
		$code = $codeArray[0];
		$codeArray = explode('<br>', $code);
		$code = $codeArray[0];

		// Remove all new-line characters
		$codeArray = explode("\n", $code);
		$code = $codeArray[0];

		// Remove carrige-return
		$code = str_replace("\n", '', $code);

	} else {
		// Should not happen!
		debug_report_bug(__FUNCTION__, __LINE__, 'Cannot parse response. Raw response:<pre>' . print_r($response, true) . '</pre>');
	}

	// Fix empty code to bad
	if (empty($code)) {
		$code = -999;
	} // END - if

	// Return error code
	return $code;
}

// [EOF]
?>
